| formant_app | R Documentation |
Starts a shiny app for manually correcting formant measurements. For more
tips, see pitch_app and http://cogsci.se/soundgen.html.
formant_app(...)
... |
presets like |
Suggested workflow: load one or several audio files (wav/mp3), preferably not longer than a minute or so. Select a region of interest in the spectrogram - for example, a sustained vowel with clear and relatively steady formants. Double-click within the selection to create a new annotation (you may add a text label if needed). If you are satisfied with the automatically calculated formant frequencies, proceed to the next region of interest. If not, there are three ways to adjust them: (1) click the spectrogram within selection (pick the formant number to adjust by clicking the formant boxes); (2) single-click the spectrum to use the cursor's position, or (3) double-click the spectrum to use the nearest spectral peak. When done with a file, move on to the next one in the queue. Use the orange button to download the results. To continue work, upload the output file from the previous session together with the audio files (you can rename it, but keep the .csv extension). Use hotkeys (eg spacebar to play/stop) and avoid working with very large files.
Recovering lost data
Every time you add an annotation or move in between files in the queue, the output you've got so far is saved in a temporary backup file. If the app crashes or is closed without saving the results, this backup file preserves your data. To recover it, restart formant_app() - a dialog box will pop up and ask whether you want to append the old data to the new one. Even so, save your data regularly to be on the safe side!
A list of the last used settings ($settings) plus a data.frame with the formant measurements. Every time a new annotation is added, the app creates a backup csv file in the session's temporary directory, and it returns the final payload upon closing the app.
pitch_app annotation_app
## Not run:
f = formant_app() # runs in default browser such as Firefox or Chrome
f1 = formant_app(specType = 'reassigned', windowLength = 5, step = 1)
# run the app with previously used settings
f2 = do.call(formant_app, f1$settings)
# save the complete output, including the settings used
saveRDS(f2, 'my_formant_analysis.rds')
# To change system default browser, run something like:
options('browser' = '/usr/bin/firefox') # path to the executable on Linux
## End(Not run)
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.