getBandwidth: Get bandwidth

View source: R/formants_utilities.R

getBandwidthR Documentation

Get bandwidth

Description

Calculates formant bandwidth as a function of formant frequencies using a modified version of TMF-63 formula. Namely, above 500 Hz it follows the original formula from Tappert, Martony, and Fant (TMF)-1963, and below 500 Hz it applies a correction to allow for energy losses at low frequencies. See Khodai-Joopari & Clermont (2002), "Comparison of formulae for estimating formant bandwidths". Below 250 Hz the bandwidth is forced to drop again to avoid very large values near zero (just guesswork!)

Usage

getBandwidth(f)

Arguments

f

a vector of formant frequencies, Hz

Examples

f = 1:5000
plot(f, soundgen:::getBandwidth(f), type = 'l',
  xlab = 'Formant frequency, Hz', ylab = 'Estimated bandwidth, Hz')

soundgen documentation built on Sept. 20, 2026, 5:07 p.m.