| getEntropy | R Documentation |
Returns Wiener or Shannon entropy of an input vector such as the spectrum of a sound. Non-positive input values are converted to a small positive number (convertNonPositive). If all elements are zero, returns NA.
getEntropy(
x,
type = c("wiener", "shannon"),
normalize = FALSE,
convertNonPositive = 1e-10
)
x |
vector of positive floats |
type |
'shannon' for Shannon (information) entropy, 'wiener' for Wiener entropy |
normalize |
if TRUE, Shannon entropy is normalized by the length of input vector to range from 0 to 1. It has no effect on Wiener entropy |
convertNonPositive |
occasional non-positive values are converted to
|
The entropy of input vector as a single number.
# Here are four simplified power spectra, each with 9 frequency bins:
s = list(
onePeak = c(rep(0, 4), 1, rep(0, 4)), # a single peak in spectrum
harmonics = c(0, 0, 1, 0, 0, .75, 0, 0, .5), # perfectly periodic, with 3 harmonics
silent = rep(0, 9), # a silent frame
noise = rep(1, 9) # white noise
)
# Wiener entropy is ~0 for periodic, NA for silent, 1 for white noise
lapply(s, function(x) round(getEntropy(x), 2))
# Shannon entropy is ~0 for periodic with a single harmonic, moderate for
# periodic with multiple harmonics, NA for silent, highest for white noise
lapply(s, function(x) round(getEntropy(x, type = 'shannon'), 2))
# Normalized Shannon entropy - same but forced to be 0 to 1
lapply(s, function(x) round(getEntropy(x,
type = 'shannon', normalize = TRUE), 2))
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.