soundgen_app: Interactive sound synthesizer

View source: R/apps.R

soundgen_appR Documentation

Interactive sound synthesizer

Description

Starts a shiny app that provides an interactive wrapper to soundgen. Note that the browser has to be able to playback WAV audio files; otherwise, there will be no sound.

Usage

soundgen_app()

Value

Does not return anything.

See Also

soundgen

Examples

## Not run: 
soundgen_app()  # opens the app in your default browser

## End(Not run)

soundgen documentation built on Sept. 20, 2026, 5:07 p.m.