| annotation_app | R Documentation |
Starts a shiny app for annotating audio. This is a simplified and faster
version of formant_app intended only for making annotations.
annotation_app(...)
... |
presets like |
A list of the last used settings ($settings) plus a data.frame with the annotations. Every time a new annotation is added, the app creates a backup csv file and creates or updates a global object called "my_annot", which contains all the annotations.
## Not run:
ann = annotation_app() # runs in default browser such as Firefox or Chrome
ann = annotation_app(specType = 'reassigned', windowLength = 5, step = 1)
# full list of parameters that can be passed to annotation_app():
# paste0(c(names(input)[which(names(input) %in% rownames(def_form))],
# 'spec_xlim', 'spec_ylim', 'spec_colorTheme', 'osc', 'wn'), collapse = ', ')
# save the complete output, including the settings used
saveRDS(ann, 'my_annotations.rds')
# To change system default browser, run something like:
options('browser' = '/usr/bin/firefox') # path to the executable on Linux
## End(Not run)
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.