if (FALSE) {
ann = annotation_app() # runs in default browser such as Firefox or Chrome
ann = annotation_app(specType = 'reassigned', windowLength = 5, step = 1)
# full list of parameters that can be passed to annotation_app():
paste0(c(rownames(soundgen:::def_ann),
'specType', 'spec_colorTheme', 'spec_yScale', 'osc', 'wn', 'audioMethod'),
collapse = ', ')
# save the complete output, including the settings used
saveRDS(ann, 'my_annotations.rds')
# re-use the same settings in a future session
ann2 = do.call(annotation_app, ann$settings)
# To change system default browser, run something like:
options('browser' = '/usr/bin/firefox') # path to the executable on Linux
}
Run the code above in your browser using DataLab