- x
path to a folder, one or more wav or mp3 files c('file1.wav',
'file2.mp3'), Wave object, numeric vector, or a list of Wave objects or
numeric vectors
- multFormants
1 = no change, >1 = raise formants (eg 1.1 = 10% up, 2 =
one octave up), <1 = lower formants. Anchor format accepted (see
soundgen)
- samplingRate
sampling rate of x (only needed if x is a
numeric vector)
- freqWindow
the width of spectral smoothing window, Hz: a single
positive number, roughly the expected spacing between harmonics. Defaults
to median detected f0
- dynamicRange
regions under -dynamicRange dB are treated as silent
- windowLength
length of the analysis window, ms
- step
step between successive windows, ms; if provided, overrides
overlap; because digital audio is sampled at discrete time intervals
of 1/samplingRate, the actual step and thus the time stamps of STFT frames
may be slightly different - e.g., 24.98866 instead of 25.0 ms
- overlap
overlap between successive windows, %
- wn
window type accepted by winFun: character string or
function
- specEnvMethod
the method of extracting a smoothed spectral envelope:
"cepstral" = Gaussian liftering of the real cepstrum (default), "gauss" =
Gaussian blur of the log spectrum, "movavg" = moving average of the log
spectrum, "peak" = moving maximum (upper envelope). See
getSpecEnv for details
- interpol
the method for interpolating scaled spectra: any method
supported by interpolate, defaults to "splineFC"
- normalize
"orig" = same as input (default), "max" = maximum possible
peak amplitude for the given input scale, "none" = no normalization
- play
if TRUE, plays the output audio using the default player on your
system. If a character string, it is passed to playme as the
name of the player to use (e.g. 'aplay', 'play', 'vlc'). In case of errors,
try setting another default player for playme
- saveAudio
if TRUE, saves the processed audio in a subdirectory named
after the function and created in the input directory (if input is a file
or folder) or in the working directory
- reportEvery
when processing multiple inputs, report estimated time
left every reportEvery iterations (NULL = default, NA = don't
report); see reportTime
- cores
number of cores for parallel processing