| soundgen_app | R Documentation |
Starts a shiny app that provides an interactive wrapper to
soundgen. Note that the browser has to be able to playback WAV
audio files; otherwise, there will be no sound.
soundgen_app()
Does not return anything.
soundgen
## Not run:
soundgen_app() # opens the app in your default browser
## End(Not run)
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.