Learn R Programming

soundgen (version 3.0.0)

soundgen_app: Interactive sound synthesizer

Description

Starts a shiny app that provides an interactive wrapper to soundgen. Note that the browser has to be able to playback WAV audio files; otherwise, there will be no sound.

Usage

soundgen_app()

Arguments

Value

Does not return anything.

See Also

soundgen

Examples

Run this code
if (FALSE) {
soundgen_app()  # opens the app in your default browser
}

Run the code above in your browser using DataLab