powered by
Starts a shiny app that provides an interactive wrapper to soundgen. Note that the browser has to be able to playback WAV audio files; otherwise, there will be no sound.
soundgen
soundgen_app()
Does not return anything.
if (FALSE) { soundgen_app() # opens the app in your default browser }
Run the code above in your browser using DataLab